==> jafuffy.0_4_9.zip: JaFuffy/Quellen/Java/jafuffy/logik/Wuerfel.java <==
/******************************************************************************
 ** $Id: Wuerfel.java 739 2015-05-23 16:13:32Z wmh $
 ** Diese Datei ist Bestandteil der Java-Quelltexte des Wrfelspiels JaFuffy.
 ** Lauffhig ab Java 6.
 ******************************************************************************
 ** Copyright (C) Wolfgang Hauck <wolfgang.hauck@3kelvin.de>
 ******************************************************************************
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************
 ** Die aktuellste Version von JaFuffy findet sich im Internet unter
 ** <http://jafuffy.3kelvin.de>.
 **
 ** Kommentare, Fehler oder Erweiterungswnsche bitte per E-Mail senden an
 ** <jafuffy@3kelvin.de>.
 ******************************************************************************/
package jafuffy.logik;

import java.util.Random;

import javax.swing.JToggleButton;
import javax.swing.event.ChangeListener;

/** Wrfelmodell. */
public class Wuerfel extends JToggleButton.ToggleButtonModel implements Comparable<Wuerfel> {

    /** Zur Serialisierung. */
    private static final long serialVersionUID = 6889772262239748227L;

    /** Zufallsgenerator. */
    private static final Random ZUFALL = new Random();

    /** Die Anzahl aller mglichen Augen. */
    public static final int AUGEN = 6;

    /** nderungen im Zustand. */
    private final Aenderungen<CEWuerfel> aenderungen;
    /** Augenzahl (von eins bis sechs). */
    private int augen;
    /** Wrfel zum erneuten Wrfeln vorschlagen? */
    private boolean vorgeschlagen;

    /**
     * Konstruktor.
     *
     * @param augen
     *            Ursprngliche Augenzahl
     */
    Wuerfel(int augen) {
        this.augen = augen;
        aenderungen = new Aenderungen<CEWuerfel>();
    }

    /**
     * ChangeListener hinzufgen.
     *
     * @param changeListener
     *            Objekt, das auf nderungsereignisse reagiert.
     */
    public void installiereChangeListener(ChangeListener changeListener) {
        aenderungen.addChangeListener(changeListener);
    }

    /**
     * @return -1,0,+1 (Vergleich)
     */
    @Override
    public int compareTo(Wuerfel wuerfel) {
        if (augen() < wuerfel.augen()) {
            return -1;
        }
        if (augen() > wuerfel.augen()) {
            return +1;
        }
        return 0;
    }

    /**
     * ChangeListener entfernen.
     *
     * @param changeListener
     *            Zuvor registriertes Beobachtungsobjekt.
     */
    public void entferneChangeListener(ChangeListener changeListener) {
        aenderungen.removeChangeListener(changeListener);
    }

    /**
     * Wrfelauswahl mit Geruscheffekten.
     *
     * @param selektiert
     *            Gibt an, ob der Wrfel ausgewhlt und damit in den Becher gelegt werden soll.
     */
    @Override
    public void setSelected(boolean selektiert) {
        boolean feuern = isSelected() ^ selektiert;
        super.setSelected(selektiert);
        if (feuern) {
            aenderungen.fireStateChanged(selektiert ? CEWuerfel.ANWAHL : CEWuerfel.ABWAHL);
        }
    }

    /**
     * @return Augenzahl von eins bis sechs
     */
    public int augen() {
        return augen;
    }

    /**
     * Wrfel zum erneuten Werfen vorschlagen.
     *
     * @param vorschlagen
     *            Gibt an, ob der Wrfel zum erneuten Werfen vorgeschlagen ist.
     */
    void setzeVorschlag(boolean vorschlagen) {
        vorgeschlagen = vorschlagen;
    }

    /** @return Zum erneuten Wrfeln vorgeschlagen? */
    public boolean vorgeschlagen() {
        return vorgeschlagen;
    }

    /** Wrfel werfen. Ein anstehender Vorschlag wird dann zurckgenommen. */
    void wirf() {
        vorgeschlagen = false;
        augen = ZUFALL.nextInt(AUGEN) + 1;
        aenderungen.fireStateChanged(CEWuerfel.WURF); // muss vor Abwahl erfolgen
        setSelected(false);
    }

}

==> jafuffy.0_5_1.zip: JaFuffy/Quellen/Java/jafuffy/logik/Wuerfel.java <==
/******************************************************************************
 ** $Id: Wuerfel.java 891 2016-02-17 01:55:44Z wmh $
 ** Diese Datei ist Bestandteil der Java-Quelltexte des Wrfelspiels JaFuffy.
 ** Lauffhig ab Java 6.
 ******************************************************************************
 ** Copyright (C) Wolfgang Hauck <wolfgang.hauck@3kelvin.de>
 ******************************************************************************
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************
 ** Die aktuellste Version von JaFuffy findet sich im Internet unter
 ** <http://jafuffy.3kelvin.de>.
 **
 ** Kommentare, Fehler oder Erweiterungswnsche bitte per E-Mail senden an
 ** <jafuffy@3kelvin.de>.
 ******************************************************************************/
package jafuffy.logik;

import java.util.Random;

import javax.swing.JToggleButton;
import javax.swing.event.ChangeListener;

/** Wrfelmodell. */
public class Wuerfel extends JToggleButton.ToggleButtonModel implements Comparable<Wuerfel> {

    /** Zur Serialisierung. */
    private static final long serialVersionUID = 6889772262239748227L;

    /** Zufallsgenerator. */
    private static final Random ZUFALL = new Random();

    /** Die Anzahl aller mglichen Augen. */
    public static final int AUGEN = 6;

    /** nderungen im Zustand. */
    private final Aenderungen<CEWuerfel> aenderungen;
    /** Augenzahl (von eins bis sechs). */
    private int augen;
    /** Wrfel zum erneuten Wrfeln vorschlagen? */
    private boolean vorgeschlagen;

    /**
     * Konstruktor.
     *
     * @param augen
     *            Ursprngliche Augenzahl
     */
    Wuerfel(int augen) {
        this.augen = augen;
        aenderungen = new Aenderungen<CEWuerfel>();
    }

    /**
     * @return Augenzahl von eins bis sechs
     */
    public int augen() {
        return augen;
    }

    /**
     * @return -1,0,+1 (Vergleich)
     */
    @Override
    public int compareTo(Wuerfel wuerfel) {
        if (augen() < wuerfel.augen()) {
            return -1;
        }
        if (augen() > wuerfel.augen()) {
            return +1;
        }
        return 0;
    }

    /**
     * ChangeListener entfernen.
     *
     * @param changeListener
     *            Zuvor registriertes Beobachtungsobjekt.
     */
    public void entferneChangeListener(ChangeListener changeListener) {
        aenderungen.removeChangeListener(changeListener);
    }

    /**
     * @param haeufigkeiten
     *            Feld mit allen Augenhufigkeiten.
     * @return Anzahl der Wrfel mit dieser Augenzahl.
     */
    public int vorkommen(int[] haeufigkeiten) {
        return haeufigkeiten[augen - 1];
    }

    /**
     * ChangeListener hinzufgen.
     *
     * @param changeListener
     *            Objekt, das auf nderungsereignisse reagiert.
     */
    public void installiereChangeListener(ChangeListener changeListener) {
        aenderungen.addChangeListener(changeListener);
    }

    /**
     * Wrfelauswahl mit Geruscheffekten.
     *
     * @param selektiert
     *            Gibt an, ob der Wrfel ausgewhlt und damit in den Becher gelegt werden soll.
     */
    @Override
    public void setSelected(boolean selektiert) {
        boolean feuern = isSelected() ^ selektiert;
        super.setSelected(selektiert);
        if (feuern) {
            aenderungen.fireStateChanged(selektiert ? CEWuerfel.ANWAHL : CEWuerfel.ABWAHL);
        }
    }

    /** @return Zum erneuten Wrfeln vorgeschlagen? */
    public boolean vorgeschlagen() {
        return vorgeschlagen;
    }

    /**
     * Wrfel zum erneuten Werfen vorschlagen.
     *
     * @param vorschlagen
     *            Gibt an, ob der Wrfel zum erneuten Werfen vorgeschlagen ist.
     */
    void setzeVorschlag(boolean vorschlagen) {
        vorgeschlagen = vorschlagen;
    }

    /** Wrfel werfen. Ein anstehender Vorschlag wird dann zurckgenommen. */
    void wirf() {
        vorgeschlagen = false;
        augen = ZUFALL.nextInt(AUGEN) + 1;
        aenderungen.fireStateChanged(CEWuerfel.WURF); // muss vor Abwahl erfolgen
        setSelected(false);
    }

}
